@@ -19,9 +19,7 @@
 #include <apt-pkg/deblistparser.h>
 #include <apt-pkg/pkgcache.h>
 #include <apt-pkg/tagfile.h>
-#include <apt-pkg/md5.h>
-#include <apt-pkg/sha1.h>
-#include <apt-pkg/sha2.h>
+#include <apt-pkg/hashes.h>
 #include <apt-pkg/init.h>
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/orderlist.h>
@@ -45,6 +43,13 @@ static char PyAptError_Doc[] =
 
 PyObject *PyAptError;
 
+static char PyAptWarning_Doc[] =
+   "Exception class for most python-apt warnings.\n"
+   "\n"
+   ".. versionadded:: 2.7";
+
+PyObject *PyAptWarning;
+
 static char PyAptCacheMismatchError_Doc[] =
    "Raised when passing an object from a different cache to\n"
    ":class:`apt_pkg.DepCache` methods\n\n"
@@ -84,8 +89,8 @@ static PyObject *VersionCompare(PyObject *Self,PyObject *Args)
 {
    char *A;
    char *B;
-   int LenA;
-   int LenB;
+   Py_ssize_t LenA;
+   Py_ssize_t LenB;
 
    if (PyArg_ParseTuple(Args,"s#s#",&A,&LenA,&B,&LenB) == 0)
       return 0;
@@ -188,7 +193,7 @@ static PyObject *RealParseDepends(PyObject *Self,PyObject *Args,PyObject *Kwds,
 
    const char *Start;
    const char *Stop;
-   int Len;
+   Py_ssize_t Len;
    const char *Arch = NULL;
    char *kwlist[] = {"s", "strip_multi_arch", "architecture", 0};
 
@@ -264,22 +269,26 @@ static PyObject *md5sum(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O",&Obj) == 0)
       return 0;
 
+   if (PyErr_WarnEx(PyExc_DeprecationWarning,
+                    "apt_pkg.md5sum is deprecated, use apt_pkg.Hashes", 1) == -1)
+        return NULL;
+
    // Digest of a string.
    if (PyBytes_Check(Obj) != 0)
    {
       char *s;
       Py_ssize_t len;
-      MD5Summation Sum;
+      Hashes Sum(Hashes::MD5SUM);
       PyBytes_AsStringAndSize(Obj, &s, &len);
       Sum.Add((const unsigned char*)s, len);
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::MD5SUM).HashValue());
    }
 
    // Digest of a file
    int Fd = PyObject_AsFileDescriptor(Obj);
    if (Fd != -1)
    {
-      MD5Summation Sum;
+      Hashes Sum(Hashes::MD5SUM);
       struct stat St;
       if (fstat(Fd,&St) != 0 ||
 	  Sum.AddFD(Fd,St.st_size) == false)
@@ -288,7 +297,7 @@ static PyObject *md5sum(PyObject *Self,PyObject *Args)
 	 return 0;
       }
 
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::MD5SUM).HashValue());
    }
 
    PyErr_SetString(PyExc_TypeError,"Only understand strings and files");
@@ -309,22 +318,26 @@ static PyObject *sha1sum(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O",&Obj) == 0)
       return 0;
 
+   if (PyErr_WarnEx(PyExc_DeprecationWarning,
+                    "apt_pkg.sha1sum is deprecated, use apt_pkg.Hashes", 1) == -1)
+        return NULL;
+
    // Digest of a string.
    if (PyBytes_Check(Obj) != 0)
    {
       char *s;
       Py_ssize_t len;
-      SHA1Summation Sum;
+      Hashes Sum(Hashes::SHA1SUM);
       PyBytes_AsStringAndSize(Obj, &s, &len);
       Sum.Add((const unsigned char*)s, len);
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA1SUM).HashValue());
    }
 
    // Digest of a file
    int Fd = PyObject_AsFileDescriptor(Obj);
    if (Fd != -1)
    {
-      SHA1Summation Sum;
+      Hashes Sum(Hashes::SHA1SUM);
       struct stat St;
       if (fstat(Fd,&St) != 0 ||
 	  Sum.AddFD(Fd,St.st_size) == false)
@@ -333,7 +346,7 @@ static PyObject *sha1sum(PyObject *Self,PyObject *Args)
 	 return 0;
       }
 
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA1SUM).HashValue());
    }
 
    PyErr_SetString(PyExc_TypeError,"Only understand strings and files");
@@ -354,22 +367,26 @@ static PyObject *sha256sum(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O",&Obj) == 0)
       return 0;
 
+   if (PyErr_WarnEx(PyExc_DeprecationWarning,
+                    "apt_pkg.sha256sum is deprecated, use apt_pkg.Hashes", 1) == -1)
+        return NULL;
+
    // Digest of a string.
    if (PyBytes_Check(Obj) != 0)
    {
       char *s;
       Py_ssize_t len;
-      SHA256Summation Sum;
+      Hashes Sum(Hashes::SHA256SUM);
       PyBytes_AsStringAndSize(Obj, &s, &len);
       Sum.Add((const unsigned char*)s, len);
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA256SUM).HashValue());
    }
 
    // Digest of a file
    int Fd = PyObject_AsFileDescriptor(Obj);
    if (Fd != -1)
    {
-      SHA256Summation Sum;
+      Hashes Sum(Hashes::SHA256SUM);
       struct stat St;
       if (fstat(Fd,&St) != 0 ||
 	  Sum.AddFD(Fd,St.st_size) == false)
@@ -378,7 +395,7 @@ static PyObject *sha256sum(PyObject *Self,PyObject *Args)
 	 return 0;
       }
 
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA256SUM).HashValue());
    }
 
    PyErr_SetString(PyExc_TypeError,"Only understand strings and files");
@@ -399,22 +416,26 @@ static PyObject *sha512sum(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O",&Obj) == 0)
       return 0;
 
+   if (PyErr_WarnEx(PyExc_DeprecationWarning,
+                    "apt_pkg.sha512sum is deprecated, use apt_pkg.Hashes", 1) == -1)
+        return NULL;
+
    // Digest of a string.
    if (PyBytes_Check(Obj) != 0)
    {
       char *s;
       Py_ssize_t len;
-      SHA512Summation Sum;
+      Hashes Sum(Hashes::SHA512SUM);
       PyBytes_AsStringAndSize(Obj, &s, &len);
       Sum.Add((const unsigned char*)s, len);
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA512SUM).HashValue());
    }
 
    // Digest of a file
    int Fd = PyObject_AsFileDescriptor(Obj);
    if (Fd != -1)
    {
-      SHA512Summation Sum;
+      Hashes Sum(Hashes::SHA512SUM);
       struct stat St;
       if (fstat(Fd,&St) != 0 ||
 	  Sum.AddFD(Fd,St.st_size) == false)
@@ -423,7 +444,7 @@ static PyObject *sha512sum(PyObject *Self,PyObject *Args)
 	 return 0;
       }
 
-      return CppPyString(Sum.Result().Value());
+      return CppPyString(Sum.GetHashString(Hashes::SHA512SUM).HashValue());
    }
 
    PyErr_SetString(PyExc_TypeError,"Only understand strings and files");
@@ -872,6 +893,9 @@ extern "C" void initapt_pkg()
    PyAptError = PyErr_NewExceptionWithDoc("apt_pkg.Error", PyAptError_Doc, PyExc_SystemError, NULL);
    if (PyAptError == NULL)
       INIT_ERROR;
+   PyAptWarning = PyErr_NewExceptionWithDoc("apt_pkg.Warning", PyAptWarning_Doc, PyExc_Warning, NULL);
+   if (PyAptWarning == NULL)
+      INIT_ERROR;
 
    PyAptCacheMismatchError = PyErr_NewExceptionWithDoc("apt_pkg.CacheMismatchError", PyAptCacheMismatchError_Doc, PyExc_ValueError, NULL);
    if (PyAptCacheMismatchError == NULL)
@@ -891,6 +915,7 @@ extern "C" void initapt_pkg()
    Config->NoDelete = true;
    PyModule_AddObject(Module,"config",Config);
    PyModule_AddObject(Module,"Error",PyAptError);
+   PyModule_AddObject(Module,"Warning",PyAptWarning);
    PyModule_AddObject(Module,"CacheMismatchError", PyAptCacheMismatchError);
 
 
